/* eslint-disable global-require */

const fs = require('fs-extra');
const { resolve } = require('path');

const testFile1 = resolve('examples/before/testFile1.js');
const testFile2 = resolve('examples/before/testFile2.js');

describe('outer work replace-in-files', () => {
  describe('testFile1', () => {
    asyncTest('"from" - Regexp and to" - Function', async () => {
      const replaceInFiles = require('../index.js');
      const files = resolve('examples/generatedAfter/testOptions.js');
      await fs.copy(testFile1, files);

      const from = /const/g;
      function to() {
        return 'var';
      }

      const options = {
        files,
        from,
        to,
      };
      const result = await replaceInFiles(options);

      const fsResult = await fs.readFile(files, 'utf8');
      const expectedResult = await fs.readFile(resolve('examples/after/testOptions.js'), 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/testOptions.js')]).toBe(7);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/testOptions.js'));
    });
    asyncTest('"to" - String', async () => {
      const replaceInFiles = require('../index.js');
      const files = resolve('examples/generatedAfter/testOptions.js');
      await fs.copy(testFile1, files);

      const from = /const/g;
      const to = 'var';

      const options = {
        files,
        from,
        to,
      };
      const result = await replaceInFiles(options);

      const fsResult = await fs.readFile(files, 'utf8');
      const expectedResult = await fs.readFile(resolve('examples/after/testOptions.js'), 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/testOptions.js')]).toBe(7);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/testOptions.js'));
    });
    asyncTest('"to" - undefined', async () => {
      const replaceInFiles = require('../index.js');
      const files = resolve('examples/generatedAfter/testOptions.js');
      await fs.copy(testFile1, files);

      const from = /const/g;

      const options = {
        files,
        from,
      };
      const result = await replaceInFiles(options);

      const fsResult = await fs.readFile(resolve('examples/generatedAfter/testOptions.js'), 'utf8');
      const expectedResult = await fs.readFile(testFile1, 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/testOptions.js')]).toBe(7);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/testOptions.js'));
    });
    asyncTest('"to" - exist and has option onlyFindPathsWithoutReplace', async () => {
      const replaceInFiles = require('../index.js');
      const files = resolve('examples/generatedAfter/testOptions.js');
      await fs.copy(testFile1, files);

      const from = /const/g;
      const to = 'var';
      const onlyFindPathsWithoutReplace = true;

      const options = {
        files,
        from,
        to,
        onlyFindPathsWithoutReplace,
      };
      const result = await replaceInFiles(options);

      const fsResult = await fs.readFile(resolve('examples/generatedAfter/testOptions.js'), 'utf8');
      const expectedResult = await fs.readFile(testFile1, 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/testOptions.js')]).toBe(7);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/testOptions.js'));
    });
    asyncTest('"from" - String', async () => {
      const replaceInFiles = require('../index.js');
      const files = resolve('examples/generatedAfter/fromIsString.js');
      await fs.copy(testFile1, files);

      const from = 'const';
      const to = 'var';

      const options = {
        files,
        from,
        to,
      };
      await replaceInFiles(options);

      const fsResult = await fs.readFile(files, 'utf8');
      const expectedResult = await fs.readFile(resolve('examples/after/fromIsString.js'), 'utf8');
      expect(fsResult).toBe(expectedResult);
    });
    asyncTest('"from" - String - with onlyFindPathsWithoutReplace option', async () => {
      const replaceInFiles = require('../index.js');
      const files = resolve('examples/generatedAfter/fromIsString.js');
      await fs.copy(testFile1, files);

      const from = 'const';
      const to = 'var';
      const onlyFindPathsWithoutReplace = true;

      const options = {
        files,
        from,
        to,
        onlyFindPathsWithoutReplace,
      };
      const result = await replaceInFiles(options);

      const fsResult = await fs.readFile(resolve('examples/generatedAfter/fromIsString.js'), 'utf8');
      const expectedResult = await fs.readFile(testFile1, 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/fromIsString.js')]).toBe(7);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/fromIsString.js'));
    });
    asyncTest('"files" - Array', async () => {
      const replaceInFiles = require('../index.js');
      const files = [
        resolve('examples/generatedAfter/testOptions.js')
      ];
      await fs.copy(testFile1, files[0]);

      const from = /const/g;
      const to = 'var';

      const options = {
        files,
        from,
        to,
      };
      const result = await replaceInFiles(options);

      const fsResult = await fs.readFile(files[0], 'utf8');
      const expectedResult = await fs.readFile(resolve('examples/after/testOptions.js'), 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/testOptions.js')]).toBe(7);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/testOptions.js'));
    });
    asyncTest('"replaceFileOnlyIfMatchRegxpInFile" - if regexp', async () => {
      const replaceInFiles = require('../index.js');
      const files = [
        resolve('examples/generatedAfter/testOptions.js')
      ];
      await fs.copy(testFile1, files[0]);

      const from = /const/g;
      const to = 'var';
      const replaceFileOnlyIfMatchRegxpInFile = /const log = {};/;

      const options = {
        files,
        from,
        to,
        replaceFileOnlyIfMatchRegxpInFile,
      };
      const result = await replaceInFiles(options);

      const fsResult = await fs.readFile(files[0], 'utf8');
      const expectedResult = await fs.readFile(resolve('examples/after/testOptions.js'), 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/testOptions.js')]).toBe(7);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/testOptions.js'));
    });
    asyncTest('"replaceFileOnlyIfMatchRegxpInFile" - if string', async () => {
      const replaceInFiles = require('../index.js');
      const files = [
        resolve('examples/generatedAfter/testOptions.js')
      ];
      await fs.copy(testFile1, files[0]);

      const from = /const/g;
      const to = 'var';
      const replaceFileOnlyIfMatchRegxpInFile = 'const log = {};';

      const options = {
        files,
        from,
        to,
        replaceFileOnlyIfMatchRegxpInFile,
      };
      const result = await replaceInFiles(options);

      const fsResult = await fs.readFile(files[0], 'utf8');
      const expectedResult = await fs.readFile(resolve('examples/after/testOptions.js'), 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/testOptions.js')]).toBe(7);
      expect(result.replaceInFilesOptions).toEqual([options]);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/testOptions.js'));
    });
  });
  describe('testFile2', () => {
    asyncTest('sequentially replaces', async () => {
      const replaceInFiles = require('../index.js');
      const files = [
        resolve('examples/generatedAfter/testSequentially.js')
      ];
      await fs.copy(testFile2, files[0]);

      const mainSettings = {
        files,
        from: 'log',
        to: 'myLog',
      };

      const result = await replaceInFiles(mainSettings)
        .pipe({ from: 'file', to: 'myFile' })
        .pipe({ from: 'created2', to: 'test42' })
        .pipe({ from: 'created1', to: 'test79' })
        .pipe({ from: 'console.log(created2);', to: 'alert("worked!")' })
        .pipe({ from: /test/g, to: 'TEST' });

      const fsResult = await fs.readFile(files[0], 'utf8');
      const expectedResult = await fs.readFile(resolve('examples/after/testSequentially.js'), 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.replaceInFilesOptions).toEqual([
        mainSettings,
        { from: 'file', to: 'myFile' },
        { from: 'created2', to: 'test42' },
        { from: 'created1', to: 'test79' },
        { from: 'console.log(created2);', to: 'alert("worked!")' },
        { from: /test/g, to: 'TEST' }
      ]);
      expect(result.countOfMatchesByPaths[0][resolve('examples/generatedAfter/testSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[1][resolve('examples/generatedAfter/testSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[2][resolve('examples/generatedAfter/testSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[3][resolve('examples/generatedAfter/testSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[4][resolve('examples/generatedAfter/testSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[5][resolve('examples/generatedAfter/testSequentially.js')]).toBe(2);
      expect(result.paths.length).toBe(1);
      expect(result.paths[0]).toBe(resolve('examples/generatedAfter/testSequentially.js'));
    });
    asyncTest('sequentially replaces if not replaces in first iteration', async () => {
      const replaceInFiles = require('../index.js');
      const files = [
        resolve('examples/generatedAfter/testCreatePathsInSequentially.js')
      ];
      await fs.copy(testFile2, files[0]);

      const mainSettings = {
        files,
        from: 'SHOULD_NOT_FIND_THIS_STRING',
        to: 'myNewString (should not replace)',
      };

      const result = await replaceInFiles(mainSettings)
        .pipe({ from: 'file', to: 'myFile' })
        .pipe({ from: 'created2', to: 'test42' })
        .pipe({ from: 'created1', to: 'test79' })
        .pipe({ from: 'console.log(created2);', to: 'alert("worked!")' })
        .pipe({ from: /test/g, to: 'TEST' });

      const fsResult = await fs.readFile(files[0], 'utf8');
      const expectedResult = await fs.readFile(resolve('examples/after/testCreatePathsInSequentially.js'), 'utf8');
      expect(fsResult).toBe(expectedResult);
      expect(result).toBeObj();
      expect(Object.keys(result).length).toBe(3);
      expect(result.replaceInFilesOptions).toEqual([
        mainSettings,
        { from: 'file', to: 'myFile' },
        { from: 'created2', to: 'test42' },
        { from: 'created1', to: 'test79' },
        { from: 'console.log(created2);', to: 'alert("worked!")' },
        { from: /test/g, to: 'TEST' }
      ]);
      expect(result.countOfMatchesByPaths[0]).toEqual({});
      expect(result.countOfMatchesByPaths[1][resolve('examples/generatedAfter/testCreatePathsInSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[2][resolve('examples/generatedAfter/testCreatePathsInSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[3][resolve('examples/generatedAfter/testCreatePathsInSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[4][resolve('examples/generatedAfter/testCreatePathsInSequentially.js')]).toBe(1);
      expect(result.countOfMatchesByPaths[5][resolve('examples/generatedAfter/testCreatePathsInSequentially.js')]).toBe(2);
      expect(result.paths.length).toBe(1);
      expect(result.paths).toEqual([resolve('examples/generatedAfter/testCreatePathsInSequentially.js')]);
    });
  });
});
