const { ADD_REPLACE_IN_FILES_OPTIONS } = require('../../lib/constants');

describe('lib/helpers.js', () => {
  test('handleError', () => {
    const helpers = require('../../lib/helpers.js');
    const err = 'err';
    expect(() => helpers.handleError(err)).toThrow(err);
  });
  test('fs', () => {
    const helpers = require('../../lib/helpers.js');
    const {
      readFile,
      stat,
      writeFile
    } = require('fs');

    jest.mock('es6-promisify');
    const { promisify } = require('es6-promisify');

    expect(Object.keys(helpers.fs)).toEqual([
      'readFile',
      'stat',
      'writeFile'
    ]);

    expect(promisify).toHaveBeenCalledTimes(3);
    expect(promisify).toHaveBeenCalledWith(readFile);
    expect(promisify).toHaveBeenCalledWith(stat);
    expect(promisify).toHaveBeenCalledWith(writeFile);
  });
  asyncTest('delay', async () => {
    const helpers = require('../../lib/helpers.js');

    let error;
    try {
      await helpers.delay();
    } catch (err) {
      error = err;
    }
    expect(error).toBeUndefined();
  });
  describe('pipe', () => {
    test('1', () => {
      const helpers = require('../../lib/helpers.js');
      helpers.eventEmitter.emit = fn();

      const srcResult = {};
      const options = 'options';
      const result = helpers.pipe(srcResult, options);

      expect(helpers.eventEmitter.emit).toHaveBeenCalledTimes(1);
      expect(helpers.eventEmitter.emit).toHaveBeenCalledWith(ADD_REPLACE_IN_FILES_OPTIONS, options);

      expect(result.pipe).toBeFunc();
    });
    test('2', () => {
      const helpers = require('../../lib/helpers.js');
      helpers.eventEmitter.emit = fn();

      const srcResult = {};
      const options = 'options';
      const options2 = 'options2';
      const preResult = helpers.pipe(srcResult, options);
      const result = preResult.pipe(options2);
      expect(helpers.eventEmitter.emit).toHaveBeenCalledTimes(2);
      expect(helpers.eventEmitter.emit).toHaveBeenCalledWith(ADD_REPLACE_IN_FILES_OPTIONS, options);
      expect(helpers.eventEmitter.emit)
        .toHaveBeenCalledWith(ADD_REPLACE_IN_FILES_OPTIONS, options2);

      expect(result.pipe).toBeFunc();
    });
    test('3', () => {
      const helpers = require('../../lib/helpers.js');
      helpers.eventEmitter.emit = fn();

      const srcResult = {};
      const options = 'options';
      const options2 = 'options2';
      const options3 = 'options3';
      const options4 = 'options4';
      const options5 = 'options5';
      const result = helpers.pipe(srcResult, options)
        .pipe(options2)
        .pipe(options3)
        .pipe(options4)
        .pipe(options5);

      expect(helpers.eventEmitter.emit).toHaveBeenCalledTimes(5);
      expect(helpers.eventEmitter.emit).toHaveBeenCalledWith(ADD_REPLACE_IN_FILES_OPTIONS, options);
      expect(helpers.eventEmitter.emit)
        .toHaveBeenCalledWith(ADD_REPLACE_IN_FILES_OPTIONS, options2);
      expect(helpers.eventEmitter.emit)
        .toHaveBeenCalledWith(ADD_REPLACE_IN_FILES_OPTIONS, options3);
      expect(helpers.eventEmitter.emit)
        .toHaveBeenCalledWith(ADD_REPLACE_IN_FILES_OPTIONS, options4);
      expect(helpers.eventEmitter.emit)
        .toHaveBeenCalledWith(ADD_REPLACE_IN_FILES_OPTIONS, options5);

      expect(result.pipe).toBeFunc();
    });
  });
});
